@@ -1,5 +1,5 @@
 package IO::Pager::Perl;
-our $VERSION = '2.00'; #Untouched since 2.00
+our $VERSION = '2.01'; #Untouched since 2.01
 
 use strict;
 use warnings;
@@ -13,8 +13,8 @@ local $SIG{INT} = local $SIG{QUIT} = \&close;
 sub ReadMode;
 sub ReadKey;
 
-
 sub new {
+  print "\e[?1000;1006;1015h";
   my $class = shift;
   my %param = @_;
   local $ENV{TERM} = $ENV{TERM} || '';
@@ -151,6 +151,7 @@ EOH
   #"\e\[1;3D"=> #M-right
 
   $me->add_func(%config,
+		"\e[<" => \&mouse,
 		'/(\d+)/' => 1,         #jump to line
 		"\e[D" => \&tab_left,   #left
 		"\e[C" => \&tab_right,  #right, 
@@ -350,6 +351,7 @@ sub more {
   }
   else{
     print $me->{NOR};
+    $DB::single = 1;
 
 
     #INPUT LOOP, revised with inspiration from Term::Screen::getch()
@@ -365,8 +367,8 @@ sub more {
       return 1 unless defined($char);
       $me->{_I18N}{prompt} = $input .= $char;
       $me->status();
-
-     unless( ($input=~ /^\e/ and index($me->{_fncRE}, $input)>0 )
+     
+      unless( ($input=~ /^\e/ and index($me->{_fncRE}, $input)>0 )
 	      || $input =~ /^\d+/
 	      || $input =~ /:+/
 	      || defined($me->{_fnc}->{$input}) ){
@@ -376,9 +378,11 @@ sub more {
       }
 
       if( $me->{_fnc}->{$input} ){
-	use B 'svref_2object';
-	my $n = $me->{_fnc}->{$input};
-	$n = svref_2object($n)->GV->NAME;
+	#Get mapped sub name
+#	use B 'svref_2object';
+#	my $n = $me->{_fnc}->{$input};
+#	$n = svref_2object($n)->GV->NAME;
+
 	$exit = $me->{_fnc}->{$input}->($me);
 	$me->{_I18N}{prompt} = $input = '';
       }
@@ -498,6 +502,7 @@ sub status{
   my $end= $me->{_cursor} + $me->{rows};
 
   my $pct = $me->{_txtN} > $end ? $end/($me->{_txtN}) : 1;
+  #XXX unify with scrollbar: consistency and as private property
   my $pos = $me->{_cursor} ?
     ($pct==1 ? $me->{_I18N}{bottom} : 'L'.$me->{_cursor}) :
 	       $me->{_I18N}{top};
@@ -508,7 +513,7 @@ sub status{
   print $me->{_term}->Tputs('ce');			# clear line
   my $minihelp = $me->{_I18N}{minihelp};
   (my $pSansCodes = $p) =~ s/\e\[[\d;]*[a-zA-Z]//g;
-  my $pN = $me->{cols} - 2 - length($pSansCodes) - length($me->{_I18N}{minihelp});
+  my $pN = $me->{cols} -1 -length($pSansCodes) -length($me->{_I18N}{minihelp});
   $p .= ' ' x ($pN > 1 ? $pN : 1);
   $minihelp = $pN>2 ? $minihelp : do {$minihelp =~ s/\000.+//; $minihelp };
   print $me->{REV};					# reverse video
@@ -518,7 +523,7 @@ sub status{
 
 sub close{
   ReadMode 0;
-  print "\n";
+  print "\n\e[?1000l";
   $| = $SP || 0;
   #Did we exit via signal or user?
   $RT ? die : return \"foo";
@@ -622,13 +627,50 @@ sub refresh{
 
 sub scrollBar{
   my $me = shift;
-  my $mark = sprintf("%i", $me->{_cursor} / $me->{_txtN} * $me->{rows});
-  my $pages = sprintf("%i", $me->{_txtN}/$me->{rows});
-  my $thumb = int($me->{rows}/$pages);
+  $me->{_pages}  = $me->{_txtN}/$me->{rows};
+  $me->{_thumbW} = $me->{rows}/$me->{_pages};
+  $me->{_thumbT} = sprintf("%i", ($me->{_cursor} / $me->{_pages}) )+($me->{_cursor}>$me->{_txtN}/2);
+  $me->{_thumbB} =   sprintf("%i",  $me->{_thumbT}+$me->{_thumbW});
+
+#$me->dialog("cursor $me->{_cursor} top $me->{_thumbT} + width $me->{_thumbW}");
 
   for my $n (0 .. $me->{rows} -1){
     print $me->{_term}->Tgoto('cm', $me->{cols}+1, $n);
-    print $n>=$mark && $n<$mark+$thumb ? ' ' : "$me->{REV} $me->{NOR}";
+    print $n>=$me->{_thumbT} && $n<$me->{_thumbB} ? ' ' : "$me->{REV} $me->{NOR}";
+  }
+}
+
+sub mouse{
+  my $me = shift;
+  my $input ='';
+  $input .= ReadKey(0) until $input =~ /M$/i;
+
+  my @args = split /;/, $input;
+
+  if( $args[0] == 65 ){
+    $me->downhalf(); }
+  elsif( $args[0] == 64 ){
+    $me->uphalf(); }
+  elsif( $me->{scrollBar} && $args[1] == $me->{cols}+1 ){
+    if( chop $args[2] eq 'm'){ #mouse-up
+      if( $me->{_thumbDrag} ){
+	$me->{_thumbDrag} = 0;
+	my $pos;
+	if( $args[2]==1 ){
+	    $pos=0 }
+	elsif( $args[2]==$me->{rows} ){
+	    $pos= $me->{_txtN} - 2*$me->{rows}-1 }
+	else{
+	    $pos = sprintf("%i", $args[2] / $me->{rows} * $me->{_txtN}) }
+	$me->jump($pos);
+      }
+      $me->uppage() if $args[2] < $me->{_thumbT};
+      $me->downpage() if $args[2] > $me->{_thumbB};
+    }
+    elsif( $args[2]>=$me->{_thumbT} &&
+	   $args[2]<=$me->{_thumbB} ){ #automagically M (mouse-down)
+      $me->{_thumbDrag}=1;
+    }
   }
 }
 
@@ -695,7 +737,7 @@ sub down_lines{
 
   LINE: for(1..$n){
     if( $me->{_end} >= $me->{_txtN}-1 ){
-      exit if $me->{eof} && ref($me->{text}) ne 'CODE';
+      $me->close() if $me->{eof} && ref($me->{text}) ne 'CODE';
       if( ref($me->{text}) eq 'CODE' ){
 	$me->add_text( $me->{text}->() ); }
       else{
@@ -785,7 +827,11 @@ sub to_top {  $_[0]->jump(0); }
 
 sub to_bott{
   my $me = shift;
-  $me->jump( $me->{rows}>$me->{_txtN} ? 0 : $me->{_txtN}-$me->{rows} );
+  if( $me->{rows}>$me->{_txtN} ){
+    $me->jump( 0 ) }
+  else{
+    $me->jump( $me->{_txtN}-1 );
+    $me->uppage() }
 }
 
 sub save_mark{
@@ -1124,6 +1170,12 @@ Pass control characters from input unadulterated to the terminal.
 By default, chracters other than tab and newline will be converted
 to caret notation e.g; ^@ for null or ^L for form feed.
 
+=item I<scrollBar> =E<gt>0
+
+=item B<--scrollbar>
+
+Display an interactive scrollbar in the right-most column.
+
 =item I<squeeze> =E<gt>0
 
 Collapse multiple blank lines into one.
